// WorkerSettings
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include "WorkerSettings.h"

// Core
#include "Core/Env/Env.h"
#include "Core/FileIO/FileStream.h"
#include "Core/Strings/AStackString.h"

// system
#include "Windows.h"

// Other
//------------------------------------------------------------------------------
#define FBUILDWORKER_SETTINGS_VERSION 1

// CONSTRUCTOR
//------------------------------------------------------------------------------
WorkerSettings::WorkerSettings()
	: m_Mode( WHEN_IDLE )
	, m_NumCPUsToUse( Env::GetNumProcessors() )
{
	Load();

	// handle CPU downgrade
	m_NumCPUsToUse = Math::Min( Env::GetNumProcessors(), m_NumCPUsToUse );
}

// DESTRUCTOR
//------------------------------------------------------------------------------
WorkerSettings::~WorkerSettings()
{
}

// SetMode
//------------------------------------------------------------------------------
void WorkerSettings::SetMode( Mode m )
{ 
	m_Mode = m;
}

// SetNumCPUsToUse
//------------------------------------------------------------------------------
void WorkerSettings::SetNumCPUsToUse( uint32_t c )
{
	m_NumCPUsToUse = c;
}

// Load
//------------------------------------------------------------------------------
void WorkerSettings::Load()
{
	AStackString<> settingsPath;
	Env::GetExePath( settingsPath );
	settingsPath += ".settings";

	FileStream f;
	if ( f.Open( settingsPath.Get(), FileStream::READ_ONLY ) )
	{
		char header[ 4 ] = { 0 };
		f.Read( &header, 4 );
		if ( header[ 3 ] > FBUILDWORKER_SETTINGS_VERSION )
		{
			return; // version is newer and cannot be read
		}

		// settings
		f.Read( (uint32_t &)m_Mode );
		f.Read( m_NumCPUsToUse );
	}
}

// Save
//------------------------------------------------------------------------------
void WorkerSettings::Save()
{
	AStackString<> settingsPath;
	Env::GetExePath( settingsPath );
	settingsPath += ".settings";

	FileStream f;
	if ( f.Open( settingsPath.Get(), FileStream::WRITE_ONLY ) )
	{
		bool ok = true;

		// header
		ok &= ( f.Write( "FWS", 3 ) == 3 );
		ok &= ( f.Write( uint8_t( FBUILDWORKER_SETTINGS_VERSION ) ) == 1 );

		// settings
		ok &= f.Write( (uint32_t)m_Mode );
		ok &= f.Write( m_NumCPUsToUse );

		if ( ok )
		{
			return;
		}
	}

	MessageBox( nullptr, "Failed to save settings.", "FBuildWorker", MB_OK );
}

//------------------------------------------------------------------------------
